#ifndef _INTERFACE_SDL
#define _INTERFACE_SDL

#include "sdl.h"

#define FONT1 "files/fonts/FiraCode-Regular.ttf"
#define FONT2 "files/fonts/FiraCode-Medium.ttf"

enum {
	TEXT_ARRAY_FREE = 1,
	TEXT_ARRAY_SCAN
};

typedef struct {
	TTF_Font *font1;
	TTF_Font *font2;
} FONTS;

typedef struct {
	SDL_Texture *texture;
	SDL_Color color;

	int w;
	int h;

	/* Status of initialization */
	int init;
} TEXT;

int TextArray(TEXT text[], size_t n, char flag);
void PrintText(TTF_Font *font, APP *app, TEXT *dst, const char *text);
void AddBanner(APP *app, TEXT text, SDL_Rect rect);
void AddPseudoButton(APP *app, TEXT text, const int bttn_num);
void CloseFonts(FONTS *fonts);
int InitFonts(FONTS *fonts, const APP app);
#endif
